Package: EQUALrepeat
Type: Package
Title: Algorithm Driven Time Series Analysis for Researchers without
        Coding Skills
Version: 0.4.5
Date: 2026-05-18
Authors@R: person("Kurinchi", "Gurusamy", role = c("aut", "cre"), email = "k.gurusamy@ucl.ac.uk")
Author: Kurinchi Gurusamy [aut, cre]
Maintainer: Kurinchi Gurusamy <k.gurusamy@ucl.ac.uk>
Depends: stringr, stats, ggplot2, DescTools
Imports: zip, cowplot, rstatix, forecast, tseries, urca, vars,
        viridisLite
Description: Support functions for R-based 'EQUAL-STATS' software which automatically classifies the data and performs appropriate statistical tests. 'EQUAL-STATS' software is a shiny application with an user-friendly interface to perform complex statistical analysis. Gurusamy,K (2024)<doi:10.5281/zenodo.13354162>.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://sites.google.com/view/equal-group/home
Note: This update removes dependencies on irr package.
NeedsCompilation: no
Packaged: 2026-05-18 08:03:39 UTC; kurin
Repository: CRAN
Date/Publication: 2026-05-18 08:50:02 UTC
Built: R 4.6.0; ; 2026-05-18 10:06:48 UTC; unix
