export(function.rbind_different_column_numbers)
export(round_near)
export(function.read_data)
export(function.Measurement_Error)
export(function.Repeated_Measures)
export(function.Time_Series)
import(stringr)
import(stats)
import(ggplot2)
import(DescTools)
importFrom(utils, read.csv, write.table)
importFrom(cowplot, plot_grid, ggdraw, draw_label, as_grob)
importFrom(zip, zip)
importFrom(rstatix, get_anova_table)
importFrom(forecast, forecast, auto.arima, findfrequency, ndiffs, nsdiffs)
importFrom(tseries, adf.test)
importFrom(urca, ca.jo)
importFrom(vars, vec2var, VAR)
importFrom(viridisLite, viridis)
